@@ -1,5 +1,5 @@
 /*---[ firestarter.c ]------------------------------------------------
- * Copyright (C) 2000-2002 Tomas Junnonen (majix@sci.fi)
+ * Copyright (C) 2000-2004 Tomas Junnonen (majix@sci.fi)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,12 +28,10 @@
 #include "util.h"
 #include "logread.h"
 #include "wizard.h"
-#include "tray.h"
 #include "preferences.h"
 #include "scriptwriter.h"
-
-#define RETURNED_EXT_FAILED 2
-#define RETURNED_INT_FAILED 3
+#include "dhcp-server.h"
+#include "statusview.h"
 
 FirestarterApp Firestarter;
 
@@ -48,24 +46,9 @@ static gint save_session (GnomeClient       *client,
 static void session_die (GnomeClient        *client,
                          gpointer            client_data);
 
-static void appbar_update (FirewallStatus status);
-static void unlock (void);
-static gboolean is_locked (void);
-static FirewallStatus current_status;
+gboolean firestarter_is_locked (void);
 
-const static gchar *
-iptables_path (void) {
-	static gchar *path = NULL;
-
-	if (path == NULL) {
-		if (access("/sbin/iptables", R_OK) == 0)
-			path = g_strdup ("/sbin/iptables");
-		else
-			path = g_strdup ("`which iptables`");
-	}
-	
-	return path;
-}
+static FirewallStatus firewall_state_prelock;
 
 /* [ stop_firewall ]
  * Flushes, zeroes and sets all policies to accept
@@ -74,45 +57,38 @@ void
 stop_firewall (void)
 {
 	gint retval;
-	const gchar *ipt;
-	gchar *error;
-	gchar *command = NULL;
+	gchar *arg[3] = {"firestarter.sh", "stop", NULL};
+	gchar *output;
+	GError *error = NULL;
 
-	ipt = iptables_path ();
+	if (g_spawn_sync (FIRESTARTER_RULES_DIR "/firestarter",
+	                  arg, NULL,
+	                  G_SPAWN_STDERR_TO_DEV_NULL,
+	                  NULL, NULL,
+	                  &output, /* Standard output */
+	                  NULL, /* Standard error */
+	                  &retval, &error) != TRUE) {
+		printf ("Error spawning shell process: %s\n", error->message);
+	}
 
-	command = g_strconcat (ipt, " -F;",
-	                       ipt, " -X;",
-	                       ipt, " -Z;",
-	                       ipt, " -P INPUT ACCEPT;",
-	                       ipt, " -P FORWARD ACCEPT;",
-	                       ipt, " -P OUTPUT ACCEPT;", NULL);
-	retval = system (command);
+	printf ("%s", output);
 
 	if (retval == 0) {
-		g_print (_("Firewall stopped, network traffic is now flowing freely\n"));
-		unlock ();
-
 		if (!CONSOLE)
-			update_status (STATUS_STOPPED);
+			status_set_state (STATUS_STOPPED);
 	} else {
 		retval = WEXITSTATUS (retval);
 	
-		error = g_strdup (_("Failed to stop the firewall\n\n"));
-		if (!CONSOLE)
-			error = g_strconcat ("<span weight=\"bold\" size=\"larger\">", error, "</span>", NULL);
-
-		error = g_strconcat (error,
-			_("There was an undetermined error when trying to stop your firewall."), NULL);
-
 		if (CONSOLE)
-			fprintf (stderr, "%s", error);
+			show_error (_("Failed to stop the firewall"));
 		else
-			error_dialog (error);
-
-		g_free (error);
+			error_dialog (_("Failed to stop the firewall"),
+			              _("Failed to stop the firewall"),
+				      _("There was an undetermined error when trying to stop the firewall."),
+				      Firestarter.window);
 	}
-
-	g_free (command);
+	
+	g_free (output);
 }
 
 /* [ start_firewall ]
@@ -122,97 +98,128 @@ void
 start_firewall (void)
 {
 	gint retval;
-	gchar *error;
-	gchar *command = g_strjoin (NULL, "/bin/sh ",
-	                            FIRESTARTER_RULES_DIR "/firestarter/firewall.sh",
-	                            NULL);
+	gchar *arg[3] = {"firestarter.sh", "start", NULL};
+	gchar *output;
+	GError *error = NULL;
 
-	retval = system (command);
+	if (g_spawn_sync (FIRESTARTER_RULES_DIR "/firestarter",
+	                  arg, NULL,
+	                  G_SPAWN_STDERR_TO_DEV_NULL,
+	                  NULL, NULL,
+	                  &output, /* Standard output */
+	                  NULL, /* Standard error */
+	                  &retval, &error) != TRUE) {
+		printf ("Error spawning shell process: %s\n", error->message);
+	}
+
+	printf ("%s", output);
 
 	if (retval == 0) {
-		g_print (_("Firewall started\n"));
-
 		if (!CONSOLE)
-			update_status (STATUS_RUNNING);
+			status_set_state (STATUS_RUNNING);
 	} else {
+		gchar *message;
 		retval = WEXITSTATUS (retval);
-	
-		error = g_strdup (_("Failed to start the firewall\n\n"));
-		if (!CONSOLE)
-			error = g_strconcat ("<span weight=\"bold\" size=\"larger\">", error, "</span>", NULL);
 
-		if (retval == RETURNED_EXT_FAILED) {
-			error = g_strconcat (error, g_strdup_printf (_(
-				"The device %s is not ready.\n\n"), 
-				preferences_get_string (PREFS_FW_EXT_IF)), NULL);
-		} else if (retval == RETURNED_INT_FAILED) {
-			error = g_strconcat (error, g_strdup_printf (_(
-				"The device %s is not ready.\n\n"), 
-				preferences_get_string (PREFS_FW_EXT_IF)), NULL);
+		if (retval == RETURN_EXT_FAILED) {
+			message = g_strdup_printf (_(
+				"The device %s is not ready."), 
+				preferences_get_string (PREFS_FW_EXT_IF));
+		} else if (retval == RETURN_INT_FAILED) {
+			message = g_strdup_printf (_(
+				"The device %s is not ready."), 
+				preferences_get_string (PREFS_FW_EXT_IF));
+		} else if (retval == RETURN_NO_IPTABLES) {
+			message = g_strdup (_("Your kernel does not support iptables."));
+		} else {
+			message = g_strdup (_("An unknown error occurred."));
 		}
 
-		error = g_strconcat (error, _(
+		message = g_strconcat (message, "\n\n", _(
 			"Please check your network device settings and make sure your\n"
 			"Internet connection is active."), NULL);
 		
-		if (CONSOLE)
-			fprintf (stderr, "%s", error);
-		else {
-			error_dialog (error);
-			update_status (STATUS_STOPPED);
+		if (CONSOLE) {
+			message = g_strconcat (_("Failed to start the firewall\n"),
+			                       message, NULL);
+			show_error (message);
+		} else {
+			error_dialog (_("Failed to start the firewall"),
+			              _("Failed to start the firewall"),
+				      message,
+				      Firestarter.window);
+			status_set_state (STATUS_STOPPED);
 		}
 
-		g_free (error);
+		g_free (message);
 	}
 
-	g_free (command);
+	g_free (output);
 }
 
-/* [ halt_firewall ]
+void
+restart_firewall_if_active (void)
+{
+	if (status_get_state () == STATUS_RUNNING ||
+	    status_get_state () == STATUS_HIT)
+			start_firewall ();
+}
+
+/* [ lock_firewall ]
  * Flushes and sets all policies to deny
  */
 void
-halt_firewall (void)
+lock_firewall (void)
 {
 	gint retval;
-	const gchar *ipt;
-	gchar *error;
-	gchar *command;
+	gchar *arg[3] = {"firestarter.sh", "lock", NULL};
+	gchar *output;
+	GError *error = NULL;
 
-	ipt = iptables_path ();
-	
-	command = g_strconcat (ipt, " -P INPUT DROP;",
-	                       ipt, " -P FORWARD DROP;",
-	                       ipt, " -P OUTPUT DROP;",
-	                       ipt, " -F;",
-	                       ipt, " -X;",
-	                       ipt, " -Z;", NULL);
-	retval = system (command);
+	firewall_state_prelock = status_get_state ();
+
+	if (g_spawn_sync (FIRESTARTER_RULES_DIR "/firestarter",
+	                  arg, NULL,
+	                  G_SPAWN_STDERR_TO_DEV_NULL,
+	                  NULL, NULL,
+	                  &output, /* Standard output */
+	                  NULL, /* Standard error */
+	                  &retval, &error) != TRUE) {
+		printf ("Error spawning shell process: %s\n", error->message);
+	}
+
+	printf ("%s", output);
 
 	if (retval == 0) {
-		g_print (_("All traffic halted\n"));
-		
 		if (!CONSOLE)
-			update_status (STATUS_HALTED);
+			status_set_state (STATUS_LOCKED);
 	} else {
 		retval = WEXITSTATUS (retval);
 	
-		error = g_strdup (_("Failed to enter halted firewall mode\n\n"));
-		if (!CONSOLE)
-			error = g_strconcat ("<span weight=\"bold\" size=\"larger\">", error, "</span>", NULL);
-
-		error = g_strconcat (error,
-			_("There was an undetermined error when trying to halt the network traffic."), NULL);
-
 		if (CONSOLE)
-			fprintf (stderr, "%s", error);
-		else
-			error_dialog (error);
-	
-		g_free (error);
+			show_error (_("Failed to lock the firewall"));
+		else {
+			error_dialog (_("Failed to lock the firewall"),
+			              _("Failed to lock the firewall"),
+				      _("There was an undetermined error when trying to lock the firewall."),
+				      Firestarter.window);
+		}
 	}
 
-	g_free (command);
+	g_free (output);
+}
+
+/* [ unlock_firewall ]
+ * Return the firewall to the state prior to locking
+ */
+void
+unlock_firewall (void)
+{
+	if (firewall_state_prelock == STATUS_RUNNING ||
+	    firewall_state_prelock == STATUS_HIT)
+		start_firewall ();
+	else
+		stop_firewall ();
 }
 
 /* [ exit_firestarter ]
@@ -221,9 +228,6 @@ halt_firewall (void)
 void
 exit_firestarter (void)
 {
-	if (preferences_get_bool (PREFS_STOP_ON_EXIT))
-		stop_firewall ();
-
 	gtk_main_quit ();
 }
 
@@ -258,39 +262,6 @@ session_die (GnomeClient *client, gpointer client_data)
 	exit_firestarter ();
 }
 
-void
-update_status (FirewallStatus status)
-{
-	if (status != STATUS_HIT)
-		appbar_update (status);
-	tray_update (status);	
-
-	current_status = status;
-}
-
-FirewallStatus
-get_current_status (void)
-{
-	return current_status;
-}
-
-/* [ appbar_update ]
- * Updates the application status bar with the status of the firewall,
- */
-static void
-appbar_update (FirewallStatus status)
-{
-	if (status == STATUS_STOPPED) {
-		gnome_appbar_set_default (GNOME_APPBAR (appbar), _("Firewall stopped"));
-	}
-	else if (status == STATUS_RUNNING) {
-		gnome_appbar_set_default (GNOME_APPBAR (appbar), _("Firewall running"));
-	}
-	else if (status == STATUS_HALTED) {
-		gnome_appbar_set_default (GNOME_APPBAR (appbar), _("All traffic halted"));
-	}
-}
-
 static const gchar *
 get_lock_file_path (void)
 {
@@ -313,54 +284,25 @@ get_lock_file_path (void)
 	return path;
 }
 
-static void
-unlock (void)
+gboolean
+firestarter_is_locked (void)
 {
-	remove (get_lock_file_path ());
-}
-
-static gboolean
-is_locked (void)
-{
-	FILE *f = fopen (get_lock_file_path (), "r");
-
-	if (f != NULL) {
-		fclose (f);
-		return TRUE;
-	} else
-		return FALSE;
-}
-
-static int
-check_status_timeout (gpointer data)
-{
-	FirewallStatus state = get_current_status ();
-
-	if (state == STATUS_HIT)
-		return TRUE;
-
-	if (is_locked ()) {
-		if (state != STATUS_HALTED)
-			update_status (STATUS_RUNNING);
-	} else
-		update_status (STATUS_STOPPED);
-
-	return TRUE;
+	return g_file_test (get_lock_file_path (), G_FILE_TEST_EXISTS);
 }
 
 static void
 show_help (void)
 {
-	gchar *help = g_strdup (_(
-		"Firestarter " VERSION "\n"
-		"\n"
-		" -s, --start            Start the firewall\n"
+	gchar *help = g_strconcat (
+		_("Firestarter"), " ", VERSION "\n\n",
+		_(" -s, --start            Start the firewall\n"
 		" -p, --stop             Stop the firewall\n"
-		"     --halt             Halt all outgoing and incoming traffic (panic mode)\n"
+		"     --lock             Lock the firewall, blocking all traffic\n"
 		"     --generate-scripts Generate firewall scripts from current configuration\n"
+		"     --start-hidden     Start Firestarter with the GUI not visible\n"
 		" -v, --version          Prints Firestarter's version number\n"
 		" -h, --help             You're looking at it\n"
-	));
+	), NULL);
 
 	fprintf (stderr, "%s", help);
 	g_free (help);
@@ -370,17 +312,14 @@ static gboolean
 is_root (void)
 {
 	if (getuid () != 0) {
-		gchar *error = g_strdup (_("Insufficient privileges\n\n"));
-		if (!CONSOLE)
-			error = g_strconcat ("<span weight=\"bold\" size=\"larger\">", error, "</span>", NULL);
-		error = g_strconcat (error, _("You must have root user privileges to use Firestarter.\n"), NULL);
-
 		if (CONSOLE)
-			fprintf (stderr, "%s", error);
+		 	show_error (_("Insufficient privileges"));
 		else
-			error_dialog (error);
+			error_dialog (_("Insufficient privileges"),
+			              _("Insufficient privileges"),
+			              _("You must have root user privileges to use Firestarter."),
+			              Firestarter.window);
 
-		g_free (error);
 		return FALSE;
 	}
 
@@ -395,7 +334,8 @@ main (int argc, char* argv[])
 {
 	GnomeClient *client;
 	gint i;
-	gboolean modal_wizard;
+	gboolean must_run_wizard;
+	gboolean show_gui = TRUE;
 
 	/* Text domain and codeset */	
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
@@ -408,36 +348,42 @@ main (int argc, char* argv[])
 
 		if (!strcmp (arg, "-s") || !strcmp(arg, "--start")) {
 			CONSOLE = TRUE;
+			gnome_program_init ("firestarter", VERSION, LIBGNOME_MODULE, 1, argv, NULL);
 			if (is_root ())
 				start_firewall ();
 			return 0;
 		} else if (!strcmp (arg, "-p") || !strcmp(arg, "--stop")) {
 			CONSOLE = TRUE;
+			gnome_program_init ("firestarter", VERSION, LIBGNOME_MODULE, 1, argv, NULL);
 			if (is_root ())
 				stop_firewall ();
 			return 0;
-		} else if (!strcmp(arg, "--halt")) {
+		} else if (!strcmp(arg, "--lock")) {
 			CONSOLE = TRUE;
+			gnome_program_init ("firestarter", VERSION, LIBGNOME_MODULE, 1, argv, NULL);
 			if (is_root ())
-				halt_firewall ();
+				lock_firewall ();
 			return 0;
 		} else if (!strcmp(arg, "--generate-scripts")) {
-			gnome_init ("firestarter", VERSION, argc, argv);
-
 			CONSOLE = TRUE;
+			gnome_program_init ("firestarter", VERSION, LIBGNOME_MODULE, 1, argv, NULL);
 			if (is_root ())
-				write_script ();
+				scriptwriter_output_scripts ();
 			return 0;
+		} else if (!strcmp(arg, "--start-hidden")) {
+			show_gui = FALSE;
 		} else if (!strcmp (arg, "-v") || !strcmp(arg, "--version")) {
 			printf ("Firestarter %s\n", VERSION);
 			return 0;
 		} else if (!strcmp (arg, "-h") || !strcmp (arg, "--help") || !strcmp(arg, "-help")) {
+			CONSOLE = TRUE;
+			gnome_program_init ("firestarter", VERSION, LIBGNOME_MODULE, 1, argv, NULL);
 			show_help ();
 			return 0;	
 		}
 	}
 
-	gnome_init ("firestarter", VERSION, argc, argv);
+	gnome_program_init ("firestarter", VERSION, LIBGNOMEUI_MODULE, argc, argv, NULL);
 
 	/* Set up the session managment */
 	client = gnome_master_client ();
@@ -446,42 +392,45 @@ main (int argc, char* argv[])
 	g_signal_connect (G_OBJECT (client), "die",
 			  G_CALLBACK (session_die), NULL);
 
-
 	/* Check that the user is root */
 	if (!is_root ())
 		return 1;
 
-	/* Check that the user is running a 2.4 or 2.5 series kernel */
-	if (!detect_netfilter ()) {
-		error_dialog (_("Firestarter requires a Linux 2.4 or later kernel to function.\n\n"
-				"A client for your system might be available at http://firestarter.sf.net"));
-		exit (1);
-	}
-
 	/* Check that a valid gconf schema is installed */
 	preferences_check_schema();
 
 	/* If we're starting Firestarter for the first time or the script is missing, create modal wizard */
-	modal_wizard = (preferences_get_bool (PREFS_FIRST_RUN) || !script_exists ());
+	must_run_wizard = (preferences_get_bool (PREFS_FIRST_RUN) || !script_exists ());
 
 	/* Creating the GUI */
-	gui_construct (!modal_wizard);
+	gui_construct ();
+	/* Attach a timeout that keeps the GUI fw status in sync with userland changes */
+	status_sync_timeout (NULL); /* Do one immediate refresh */
+	g_timeout_add (5000, status_sync_timeout, NULL);
 
 	/* Initialize the system log file polling function */
 	open_logfile ((gchar *)get_system_log_path ());
 
+	if (preferences_get_bool (PREFS_FIRST_RUN))
+		policyview_install_default_ruleset ();
 
-	if (modal_wizard)
+	/* Run wizard, without the main gui visible */
+	if (must_run_wizard)
 		run_wizard ();
 	else {
-		if (preferences_get_bool (PREFS_START_ON_STARTUP))
+		/* Test that our scripts are up to date */
+		if (!scriptwriter_versions_match ()) {
+			printf (_("Updating firewall to new version...\n"));
+			scriptwriter_output_scripts ();
+			printf (_("Firewall update complete\n"));
+		}
+			
+		if (preferences_get_bool (PREFS_START_ON_GUI))
 			start_firewall ();
 
+		gui_set_visibility (show_gui);
 	}
 
-	/* Attach a timeout that updates the GUI with the true status of the firewall */
-	gtk_timeout_add (5000, check_status_timeout, NULL);
-
 	gtk_main ();
 
 	return 0;
