@@ -20,6 +20,7 @@ typedef struct _Hit  Hit;
 struct _Hit
 {
 	gchar *time;
+	gchar *direction;
 	gchar *in;
 	gchar *out;
 	gchar *port;
@@ -31,51 +32,13 @@ struct _Hit
 	gchar *service;
 };
 
-typedef struct _Rule Rule;
-
-struct _Rule
-{
-	gint type;
-	gchar *host;
-	gchar *port;
-	gboolean active;
-	gpointer extension;
-};
-
-typedef struct _Ex_Forward Forward;
-
-struct _Ex_Forward
-{
-	gchar *int_host;
-	gchar *int_port;
-};
-
-typedef enum
-{
-	STATUS_NONE,
-	STATUS_STOPPED,
-	STATUS_RUNNING,
-	STATUS_HALTED,
-	STATUS_HIT
-} FirewallStatus;
-
-typedef enum
-{
-	RULETYPE_GROUP,
-	RULETYPE_TRUSTED_HOST,
-	RULETYPE_BLOCKED_HOST,
-	RULETYPE_FORWARD,
-	RULETYPE_OPEN_PORT,
-	RULETYPE_STEALTHED_PORT,
-	RULETYPE_BLOCKED_PORT,
-	NUM_RULETYPES
-} RuleType;
+gboolean firestarter_is_locked (void);
 
 void stop_firewall (void);
 void start_firewall (void);
-void halt_firewall (void);
+void restart_firewall_if_active (void);
+void lock_firewall (void);
+void unlock_firewall (void);
 void exit_firestarter (void);
-void update_status (FirewallStatus status);
-FirewallStatus get_current_status (void);
 
 #endif
