@@ -16,18 +16,41 @@
 #include <gnome.h>
 #include "firestarter.h"
 
-void gui_construct (gboolean visible);
+void gui_construct (void);
 gboolean gui_toggle_visibility (void);
+void gui_set_visibility (gboolean visible);
 
 void show_about (GtkWidget *widget, gpointer data);
 
+/* Defintion of a GtkTreeView column */
+typedef struct _View_col View_col;
+struct _View_col
+{
+	gchar *label;
+	GType type;
+	gboolean visible;
+};
+
+/* Defintion of a GtkTreeView from which the model */
+typedef struct _View_def View_def;
+struct _View_def
+{
+	gint num_columns;
+	View_col columns[6];
+};
+
+GtkWidget *gui_create_list_view (View_def *def, gint width, gint height);
+
 typedef enum
 {
-	HITVIEW_PAGE,
-	RULEVIEW_PAGE,
+	STATUS_VIEW,
+	EVENTS_VIEW,
+	POLICY_VIEW,
 	NUM_VIEWS
 } FirestarterView;
 
+void gui_widget_sensitivity_sync (GtkToggleButton *source, GtkWidget *target);
+
 FirestarterView gui_get_active_view (void);
 
 #endif
