@@ -1,5 +1,5 @@
 /*---[ tray.h ]---------------------------------------------------------
- * Copyright (C) 2002 Tomas Junnonen (majix@sci.fi)
+ * Copyright (C) 2002-2004 Tomas Junnonen (majix@sci.fi)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,9 +14,11 @@
 
 #include <config.h>
 #include <gnome.h>
-#include "firestarter.h"
+#include "statusview.h"
 
 void tray_init (void);
 void tray_update (FirewallStatus state);
+gboolean tray_is_running (void);
+void tray_remove (void);
 
 #endif
